package db.entity;

import db.Globals;
import io.netty.buffer.ByteBuf;
import io.netty.buffer.Unpooled;
import lombok.Data;

import java.util.Arrays;

@Data
public class LogFileMeta {
    private int magic = Globals.LOG_FILE_MAGIC;
    private Integer id;
    private int[] indices;
    private long minTime;
    private long maxTime;
    private String fileName;
    private Integer count;
    private Integer valid;
    private int bodyLength;
    private int headerExtLength;

    public LogFileMeta(Integer id, String fileName) {
        this.id = id;
        this.fileName = fileName;
        this.indices = new int[Globals.MAX_LOG_ITEM];
        Arrays.fill(this.indices, -1);
        this.minTime = 0;
        this.maxTime = 0;
        this.count = 0;
        this.bodyLength = 0;
        this.headerExtLength = 0;
        this.valid = 1;
    }

    public byte[] getPayload() {
        ByteBuf buf = Unpooled.buffer(Globals.MAX_LOG_ITEM * 4 + 28 + headerExtLength);
        buf.writeInt(magic);
        for (int i = 0; i < Globals.MAX_LOG_ITEM; i++) {
            buf.writeInt(indices[i]);
        }
        buf.writeLong(minTime);
        buf.writeLong(maxTime);
        buf.writeInt(bodyLength);
        buf.writeInt(headerExtLength);
        return buf.array();
    }

    public void setPayload(byte[] buffer) {
        ByteBuf buf = Unpooled.buffer(buffer.length);
        buf.writeBytes(buffer);

        magic = buf.readInt();

        for (int i = 0; i < Globals.MAX_LOG_ITEM; i++) {
            indices[i] = buf.readInt();
        }

        minTime = buf.readLong();
        maxTime = buf.readLong();
        bodyLength = buf.readInt();
        headerExtLength = buf.readInt();
    }
}
