package db.entity;

import db.Util;
import lombok.Data;

import java.text.ParseException;
import java.util.Date;
import java.util.concurrent.atomic.AtomicLong;

/**
 * 功能描述
 *
 * @author: scott
 * @date: 2022年12月28日 2:50 PM
 */
@Data
public class Statistic {
    long beginTime;
    AtomicLong dataCount = new AtomicLong();
    AtomicLong dataCountTsdbSuccess = new AtomicLong();
    AtomicLong dataCountDiskSuccess = new AtomicLong();

    public Statistic() {
        beginTime = System.currentTimeMillis();
    }

    public long incrDataCount() {
        return this.dataCount.incrementAndGet();
    }

    public long increDataCountTsdbSuccess() {
        return this.dataCountTsdbSuccess.incrementAndGet();
    }

    public long increDataCountDiskSuccess() {
        return this.dataCountDiskSuccess.incrementAndGet();
    }

    public double dataRate() {
        long time = System.currentTimeMillis()-this.beginTime;
        return dataCount.get()*1000.0/time;
    }

    @Override
    public String toString() {
        long time = System.currentTimeMillis()-this.beginTime;
        double dataRate = dataCount.get()*1000.0/time;
        double dataCountTsdbSuccessRate = dataCountTsdbSuccess.get()*1000.0/time;
        double dataCountDiskSuccessRate = dataCountDiskSuccess.get()*1000.0/time;

        String beginTimeString = "";
        try {
            beginTimeString = Util.uTCMilliSecondsToDateString(this.beginTime);
        } catch (ParseException e) {
        }
        return "Statistic{" +
                "beginTime=" + beginTimeString +
                ", runningTime(ms)=" + time +
                ", runningTime(s)=" + time/1000 +
                ", dataCount=" + dataCount +
                ", dataCountTsdbSuccess=" + dataCountTsdbSuccess +
                ", dataCountDiskSuccess=" + dataCountDiskSuccess +
                ", dataRate=" + dataRate +
                ", dataCountTsdbSuccessRate=" + dataCountTsdbSuccessRate +
                ", dataCountDiskSuccessRate=" + dataCountDiskSuccessRate +
                '}';
    }
}
