package db.service;

import db.Globals;
import db.entity.Probe;
import db.entity.packages.ListenRequestPackage;
import db.entity.packages.ScanRequestPackage;
import db.netty.NettyServer;
import db.util.IpKit;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
@Slf4j
public class ProbeService {

    private Map<Short, Probe> probes = new Hashtable<>();

    @Autowired
    private NettyServer server;

    public Map<Short, Probe> getProbes() {
        return probes;
    }

    public void listenConfig(String typeStr, String modeStr) {

//        byte mode;
//        if (modeStr.equalsIgnoreCase("all")) {
//            mode = (byte) 0xFF;
//        } else if (modeStr.equalsIgnoreCase("black")) {
//            mode = 0x55;
//        } else if (modeStr.equalsIgnoreCase("white")) {
//            mode = (byte) 0xAA;
//        } else {
//            log.error("未识别的监听请求模式：{}", modeStr);
//            return;
//        }
//
//
//        if (typeStr.equalsIgnoreCase("Start")) {
//            listenPackageSender.startListen(mode);
//        } else {
//            listenPackageSender.cancelListen(mode);
//        }
    }

    public void scanProbes() {
        // 广播发送报文

        ScanRequestPackage scanRequestPackage = new ScanRequestPackage();

        scanRequestPackage.setToHostPort(Globals.BROADCAST_HOST, Globals.BROADCAST_PORT);

        // 仅发送一次
        scanRequestPackage.setSendTimes(1);

        log.info("<== [设备扫描] 发送{} BROADCAST_HOST:{}  BROADCAST_PORT:{}", scanRequestPackage,
                Globals.BROADCAST_HOST, Globals.BROADCAST_PORT);
        server.send(scanRequestPackage);
    }

    public String queryProbes() {
        return probes.toString();
    }

    @Scheduled(fixedDelay = 4000) // 毫秒
    private void sendListenPackage() {
        for(Probe probe: probes.values()) {
            // 发送监听报文
            ListenRequestPackage listenRequestPackage = new ListenRequestPackage(Globals.OPERATION_LISTEN_LOGIN, (byte) 0xFF);

            listenRequestPackage.setToHostPort(probe.getIp(), Globals.BROADCAST_PORT);

            log.info("向probe:{} 发送listenRequestPackage:{}", probe.getIp(), listenRequestPackage);
            server.send(listenRequestPackage);
        }

    }
    public void sendUnListenPackage(String probeIP) {
        int ipInteger;

        try {
            ipInteger = IpKit.ipToInt(probeIP);
        } catch (Exception e) {
            e.printStackTrace();
            return;
        }

        short probeID = -1;
        for(Probe probe: probes.values()) {
            if(probe.getIp()==ipInteger) {
                probeID = probe.getId();
                log.info("在probe列表中查找到probe:{} probeIP:{}", probe, probeIP);
            }
        }

        if(probeID==-1) {
            log.info("在probe列表中未查找到probeIP:{}", probeIP);
        } else {
            log.info("在probe列表中查找到probeIP:{} ", probeIP);
            probes.remove(probeID);
            log.info("在probe列表中删除probeID:{} ", probeID);
        }

        // 发送取消监听报文
        ListenRequestPackage unlistenRequestPackage = new ListenRequestPackage(Globals.OPERATION_LISTEN_CANCEL, (byte) 0xFF);

        unlistenRequestPackage.setToHostPort(probeIP, Globals.BROADCAST_PORT);

        log.info("向probe:{} 发送取消listenRequestPackage:{}", probeIP, unlistenRequestPackage);
        server.send(unlistenRequestPackage);
    }


}
