package db.entity.packages;

import db.Globals;
import db.util.ConvertUtils;
import io.netty.buffer.ByteBuf;
import io.netty.buffer.Unpooled;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.extern.slf4j.Slf4j;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Date;

@Data
@NoArgsConstructor
@Slf4j
public class DataResponsePackage extends BasePackage {
    private short probeId;
    private byte port;

    private byte dataStrategy;
    private byte clockSync;
    private short year;
    private short day;
    private short hour;
    private short minute;
    private short second;
    private int microSecond;
    private long timestamp;

    private String sMac;
    private String dMac;
    private String sIp;
    private String dIp;
    private String sPort;
    private String dPort;
    private String transportProtocolType;
    private String ipProtocolType;

    private byte[] data;

    public DataResponsePackage(short responseCode,short packageCode, short probeId, byte port, byte dataStrategy,
                               byte clockSync, short year, short day, short hour, short minute, short second,
                               int microSecond, byte[] data) {
        super(Globals.TYPE_RESPONSE, responseCode, Globals.OPERATION_LISTEN_DATA, packageCode, (short) (data.length + 20));
        this.probeId = probeId;
        this.port = port;
        this.dataStrategy = dataStrategy;
        this.clockSync = clockSync;
        this.year = year;
        this.day = day;
        this.hour = hour;
        this.minute = minute;
        this.second = second;
        this.microSecond = microSecond;
        this.data = data;
        data2Meta();
    }

    @Override
    public String toString() {
        return "DataPackage{" +
                "probeId=" + probeId +
                ", port=" + port +
                ", dataStrategy=" + Integer.toHexString(dataStrategy) +
                ", clockSync=" + Integer.toHexString(clockSync) +
                ", year=" + year +
                ", day=" + day +
                ", hour=" + hour +
                ", minute=" + minute +
                ", second=" + second +
                ", microSecond=" + microSecond +
                ", sMac=" + sMac +
                ", dMac=" + dMac +
                ", sIp=" + sIp +
                ", dIp=" + dIp +
                ", sPort=" + sPort +
                ", dPort=" + dPort +
                ", transportProtocolType=" + transportProtocolType +
                ", ipProtocolType=" + ipProtocolType +
                ", timestamp=" + timestamp +
                ", data=" + ConvertUtils.bytes2HexArray(data) +
                '}';
    }


    @Override
    public void setPayload(ByteBuf buf) throws ParseException {
        probeId = buf.readShort();
        port = buf.readByte();
        dataStrategy = buf.readByte();
        buf.readByte();
        clockSync = buf.readByte();
        year = buf.readShort();
        day = buf.readShort();
        hour = buf.readShort();
        minute = buf.readShort();
        second = buf.readShort();
        microSecond = buf.readInt();
        data = new byte[buf.readableBytes()];
        buf.readBytes(data);

        data2Meta();
    }

    @Override
    public ByteBuf getPayload() {
        ByteBuf buf = Unpooled.buffer(18 + data.length);
        buf.writeShort(probeId);
        buf.writeByte(port);
        buf.writeByte(dataStrategy);
        buf.writeByte(0);
        buf.writeByte(clockSync);
        buf.writeShort(year);
        buf.writeShort(day);
        buf.writeShort(hour);
        buf.writeShort(minute);
        buf.writeShort(second);
        buf.writeInt(microSecond);
        buf.writeBytes(data);
        return buf;
    }

    private long phraseTimeStamp() throws ParseException {
        String time = String.format("%d-%d %d:%d:%d", year, day, hour, minute, second);
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-DD HH:mm:ss");
        Date date = simpleDateFormat.parse(time);
        return (date.getTime() * 1000 + microSecond)/1000; // TODO 支持微秒
    }

    private void data2Meta() {
        byte[] sMacByte = Arrays.copyOfRange(data, 6, 12);
        byte[] dMacByte = Arrays.copyOfRange(data, 0, 6);
        byte[] ipType = Arrays.copyOfRange(data, 12, 14);
        ipProtocolType = ConvertUtils.ipProtocolType(ipType);

        byte protocol;
        byte[] sIpByte;
        byte[] dIpByte;
        if("ARP".equals(ipProtocolType)) {
            sIpByte = Arrays.copyOfRange(data, 28, 32);
            dIpByte = Arrays.copyOfRange(data, 38, 42);
            sPort = "";
            dPort = "";
            transportProtocolType = "";
        }
        else {
            sIpByte = Arrays.copyOfRange(data, 26, 30);
            dIpByte = Arrays.copyOfRange(data, 30, 34);
            byte[] sPortByte = Arrays.copyOfRange(data, 34, 36);
            byte[] dPortByte = Arrays.copyOfRange(data, 36, 38);
            sPort = String.valueOf(ConvertUtils.bytes2Short(sPortByte));
            dPort = String.valueOf(ConvertUtils.bytes2Short(dPortByte));
            protocol = data[23];
            transportProtocolType = ConvertUtils.transportProtocolType(protocol);
        }

        sMac = ConvertUtils.longMac2String(ConvertUtils.macBytes2Long(sMacByte));
        dMac = ConvertUtils.longMac2String(ConvertUtils.macBytes2Long(dMacByte));
        sIp = ConvertUtils.ip2String(ConvertUtils.bytes2Int(sIpByte));
        dIp = ConvertUtils.ip2String(ConvertUtils.bytes2Int(dIpByte));

        try {
            timestamp = phraseTimeStamp();
        } catch (ParseException e) {
            log.warn("转换时间失败：{}",this);
        }
    }

}
