package db.service;

import db.*;
import db.entity.LogFileMeta;
import db.mapper.LogFileMetaMapper;
import db.util.ConvertUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import java.io.IOException;
import java.text.ParseException;
import java.util.*;

@Service
@Slf4j
public class QueryService {

    @Autowired
    private LogFileService logFileService;

    @Autowired
    private LogFileMetaMapper logFileMetaMapper;

    @Value("${reso-db.host}")
    private String host;

    @Value("${reso-db.port}")
    private int port;

    @Value("${reso-db.metric-name}")
    private String metricName;

    private DBApiEntry entry;

    @PostConstruct
    public void afterCons() {
        this.entry = DBApiEntry.initApiEntry(host, port);
    }

    public List<byte[]> queryByTimeAndCondition(String start, String end, Map<String, String> tags) throws ParseException,
            IOException {

        List<byte[]> res = new ArrayList<>();
        Point point = new Point(metricName, tags);

        long startTime = Util.dateStringToUTCMilliSeconds(start);
        long endTime = Util.dateStringToUTCMilliSeconds(end);

        DBApiEntry entry = DBApiEntry.initApiEntry(host, port);

        List<DBVal> lists = entry.getHistRaw(point, startTime, endTime);
        log.info("queryByTimeAndCondition，查询条件为：start:{}, end:{}, point:{}", startTime, endTime, point);
        log.info("查询ResoDB，查询到：{}", lists);

        // 根据list查对应报文位置
        List<Long> locs = new ArrayList<>();
        for (DBVal dbVal : lists) {
            locs.add(Double.doubleToRawLongBits(dbVal.getValue()));
        }
        Collections.sort(locs);
        for (Long longIndex : locs) {
            byte[] log = logFileService.queryLog(longIndex);
            res.add(log);
        }
        log.info("查询磁盘，查询到：{}", res);
        return res;
    }

    public List<byte[]> queryByTime(String start, String end) throws ParseException, IOException {
        List<byte[]> res = new ArrayList<>();

        long startTime = Util.dateStringToUTCMilliSeconds(start);
        long endTime = Util.dateStringToUTCMilliSeconds(end);

        log.info("queryByTime，查询条件为：start:{}, end:{}", startTime, endTime);

        List<LogFileMeta> logFileMetas = logFileMetaMapper.getLogFileMetaBetweenTime(startTime, endTime);
        log.info("queryByTime，查询B+树，查询到：{}", logFileMetas);
        for (LogFileMeta logFileMeta : logFileMetas) {
            List<byte[]> bytes = logFileService.queryLogBetweenTime(logFileMeta, startTime, endTime);
            res.addAll(bytes);
        }
        log.info("查询磁盘，查询到：{}", res);
        return res;
    }
}
